<?php
/* For licensing terms, see /license.txt */

namespace Chamilo\PageBundle\Controller;

use Chamilo\PageBundle\Entity\Page;
use Chamilo\PageBundle\Entity\Snapshot;
use Doctrine\DBAL\Connection;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;

/**
 * Class PageController
 * @package Chamilo\PageBundle\Controller
 */
class PageController extends Controller
{
    /**
     * @Route("/cms/page/latest/{number}")
     * @param int $number
     */
    public function getLatestPages($number, Request $request)
    {
        $locale = $request->get('_locale');
        $site = $this->container->get('sonata.page.site.selector.host_by_locale')->retrieve();
        $translator = $this->get('translator');

        $criteria = [
            'enabled' => 1,
            'site' => $site,
            'decorate' => 1,
            'routeName' => 'page_slug',
            'metaKeyword' => null,
        ];
        $order = ['createdAt' => 'desc'];
        // Get latest pages
        $pages = $this->container->get('sonata.page.manager.page')->findBy($criteria, $order, $number);
        $pagesToShow = [];

        /** @var Page $page */
        foreach ($pages as $page) {
            // Skip homepage
            if ($page->getUrl() === '/') {
                continue;
            }

            $criteria = ['pageId' => $page];
            /** @var Snapshot $snapshot */
            // Check if page has a valid snapshot
            $snapshot = $this->container->get('sonata.page.manager.snapshot')->findEnableSnapshot($criteria);
            if ($snapshot) {
                $page->setMetaDescription($translator->trans($page->getMetaDescription(), [], null, $locale));
                $pagesToShow[] = $page;
            }
        }

        return $this->render(
            '@ChamiloPage/latest.html.twig',
            [ 'pages' => $pagesToShow ]
        );
    }

    /**
     * @Route("/cms/page/blocks/{number}")
     * @param int $number
     */
    public function getLatestBlocks($number, Request $request)
    {
        $site = $this->container->get('sonata.page.site.selector')->retrieve();
        $locale = $request->get('_locale');
        $translator = $this->get('translator');

        $criteria = [
            'enabled' => 1,
            'site' => $site,
            'decorate' => 1,
            'routeName' => 'page_slug',
            'metaKeyword' => 'block',
        ];
        $order = ['createdAt' => 'desc'];
        // Get latest pages
        $pages = $this->container->get('sonata.page.manager.page')->findBy($criteria, $order, $number);
        $pagesToShow = [];
        /** @var Page $page */
        foreach ($pages as $page) {
            // Skip homepage
            if ($page->getUrl() === '/') {
                continue;
            }
            $criteria = ['pageId' => $page];
            /** @var Snapshot $snapshot */
            // Check if page has a valid snapshot
            $snapshot = $this->container->get('sonata.page.manager.snapshot')->findEnableSnapshot($criteria);
            if ($snapshot) {
                $page->setMetaDescription($translator->trans($page->getMetaDescription(), [], null, $locale));
                $pagesToShow[] = $page;
            }
        }

        return $this->render(
            '@ChamiloPage/blocks.html.twig',
            [ 'pages' => $pagesToShow ]
        );
    }
}